@@ -30,6 +30,8 @@
 #include "import_plugins.h"
 #endif
 
+#include <openssl/opensslv.h>
+
 class CMSut : public QObject
 {
     Q_OBJECT
@@ -252,7 +254,9 @@ void CMSut::signverify()
             msg.waitForFinished(-1);
             QVERIFY(msg.wasSigned());
             QVERIFY(msg.success());
+#if OPENSSL_VERSION_NUMBER < 0x1010109fL
             QEXPECT_FAIL("empty", "We don't seem to be able to verify signature of a zero length message", Continue);
+#endif
             QVERIFY(msg.verifySuccess());
 
             msg.reset();
@@ -264,7 +268,9 @@ void CMSut::signverify()
             msg.waitForFinished(-1);
             QVERIFY(msg.wasSigned());
             QVERIFY(msg.success());
+#if OPENSSL_VERSION_NUMBER < 0x1010109fL
             QEXPECT_FAIL("empty", "We don't seem to be able to verify signature of a zero length message", Continue);
+#endif
             QVERIFY(msg.verifySuccess());
 
             msg.reset();
@@ -277,6 +283,9 @@ void CMSut::signverify()
             msg.waitForFinished(-1);
             QVERIFY(msg.wasSigned());
             QVERIFY(msg.success());
+#if OPENSSL_VERSION_NUMBER >= 0x1010109fL
+            QEXPECT_FAIL("empty", "On newer openssl verifaction of zero length message always succeeds", Continue);
+#endif
             QCOMPARE(msg.verifySuccess(), false);
 
             msg.reset();
@@ -490,7 +499,11 @@ void CMSut::signverify_message_invalid()
 
             // This is just to break things
             // signedResult1[30] = signedResult1[30] + 1;
-            signedResult1[signedResult1.size() - 2] = 0x00;
+            if (signedResult1.at(signedResult1.size() - 2) != 0) {
+                signedResult1[signedResult1.size() - 2] = 0x00;
+            } else {
+                signedResult1[signedResult1.size() - 2] = 0x01;
+            }
 
             msg.startVerify();
             msg.update(signedResult1);
